﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;   ///< To use Toradex C Win32 DLLs

namespace gpio_vcsharp_demo
{
    public partial class Form1 : Form
    {
        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]  ///< Importing DLL
        public static extern bool SetPinAltFn(Int32 pinNum, Int32 altFn, bool dirOut); ///< External function declaration
        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]  ///< Importing DLL
        public static extern bool GetPinLevel(Int32 pinNum); ///< External function declaration
        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern void InitGPIOLib();
        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern bool SetPinLevel(Int32 gpioNum, Int32 val);
        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern void DeInitGPIOLib();

        int sodimmNumber = 0;
        public Form1()
        {
            InitializeComponent();
            ///< GUI button and timer code 
            btnOutput.Enabled = false;
            btnInput.Enabled = false;
            btnOn.Enabled = false;
            btnOff.Enabled = false;
            btnDeinit.Enabled = false;
            tmrBtnLevel.Enabled = false;
        }
        private void FormClose(object sender, EventArgs e)
        {
            
        }
        /// <summary>
        /// Button event will set respective sodimm number pin as alternate function to GPIO
        /// </summary>
        private void btnSetGpio_Click_1(object sender, EventArgs e)
        {
            if (txtPin.TextLength == 0)
            {
                MessageBox.Show("Please Enter Valid sodimm Number");
                return;
            }
            else
            {
                sodimmNumber = int.Parse(txtPin.Text);
                SetPinAltFn(sodimmNumber, -1, true); ///< Sets SODIMM PIN ad GPIO and direction output as default
            }
            ///< UI funtion Enabled all button
            btnOutput.Enabled = true;
            btnInput.Enabled = true;
            btnDeinit.Enabled = true;
            btnSetGpio.Enabled = false;
        }
        /// <summary>
        /// setting pin direction as output
        /// </summary>
        private void btnOutput_Click(object sender, EventArgs e)
        {
            SetPinAltFn(sodimmNumber, -1, true);    ///< Sets SODIMM PIN ad GPIO and direction output
            ///< GUI code
            btnInput.Enabled = false;
            btnOn.Enabled = true;
            btnOff.Enabled = true;
        }
        /// <summary>
        /// setting pin direction as input
        /// </summary>
        private void btnInput_Click(object sender, EventArgs e)
        {
            SetPinAltFn(sodimmNumber, -1, false);   ///< Sets SODIMM PIN ad GPIO and direction input
            ///< GUI code
            btnOutput.Enabled = false;
            btnOn.Enabled = false;
            btnOff.Enabled = false;
            tmrBtnLevel.Enabled = true;
        }
        /// <summary>
        /// set gpio logic level to high
        /// </summary>
        private void btnOn_Click(object sender, EventArgs e)
        {
            SetPinLevel(sodimmNumber, 1);           ///< Sets SODIMM PIN as high when on is pressed
        }
        /// <summary>
        /// Set gpio logic level to low
        /// </summary>
        private void btnOff_Click(object sender, EventArgs e)
        {
            SetPinLevel(sodimmNumber, 0);           ///< Sets SODIMM PIN as low when off is pressed
        }
        /// <summary>
        /// DeInit GPIO library
        /// </summary>
        private void btnDeinit_Click(object sender, EventArgs e)
        {
            DeInitGPIOLib();                        ///< De-initializes GPIO library and release system resources upon closing the program
                                                    
            ////// GUI code
            btnDeinit.Enabled = false;
            btnOutput.Enabled = false;
            btnInput.Enabled = false;
            btnOn.Enabled = false;
            btnOff.Enabled = false;
            btnSetGpio.Enabled = true;
            tmrBtnLevel.Enabled = false;
            lblPinLevl.Text = "".ToString();
        }
        /// <summary>
        /// Get gpio level loop
        /// </summary>
        private void tmrBtnLevel_Tick(object sender, EventArgs e)
        {
            bool level = false;
            level = GetPinLevel(sodimmNumber);      ///< Get level of repective SODIMM PIN
            if (level)
                lblPinLevl.Text = "High";
            else
                lblPinLevl.Text = "Low";
        }
    }
}